import java.io.IOException ;

public class UnixSocket {
	
	static {
		System.loadLibrary("unixsocket") ;
	}
	
	private String addr ;
	private int sock ;
	
	public UnixSocket(String addr, int sock) {
		this.addr = addr ;
		this.sock = sock ;
	}
	
	public UnixSocket(String addr) throws IOException {
		this(addr, open(addr)) ;
	}
	
	private static int open(String addr) throws UnixSocketException {
		return JNIUtils.decode(_open(addr)) ;
	}
	
	private static native long _open(String addr) ;
	
	public String getAddress() {
		return addr ;
	}
	
	public int read(byte[] buf, int off, int len) throws IOException {
		if ((off + len) > buf.length) throw new IllegalArgumentException("length too long: " + len) ;
		return JNIUtils.decode(_read(sock, buf, off, len)) ;
	}
	
	private native static long _read(int sock, byte[] buf, int off, int len) ;
	
	public int read(byte[] buf) throws IOException {
		return read(buf, 0, buf.length) ;
	}
	
	public int read() throws IOException {
		byte[] buf = new byte[1] ;
		int n = read(buf, 0, 1) ;
		if (n != 1) throw new IOException("could not read byte") ;
		return (buf[0]) & 0xff ;
	}
	
	public int write(byte[] buf, int off, int len) throws IOException {
		if ((off + len) > buf.length) throw new IllegalArgumentException("length too long: " + len) ;
		return JNIUtils.decode(_write(sock, buf, off, len)) ;
	}
	
	private native static long _write(int sock, byte[] buf, int off, int len) ;
	
	public int write(byte[] buf) throws IOException {
		return write(buf, 0, buf.length) ;
	}
	
	public void write(int b) throws IOException {
		int written = write(new byte[] {(byte)b}, 0, 1) ;
		if (written != 1) throw new IOException("could not write byte") ;
	}
	
	public void close() throws IOException {
		JNIUtils.decode(_close(sock)) ;
	}
	
	private static native long _close(int sock) ;
	
}
